#!/usr/bin/gawk -f

function printEventRow(class, uid, event)
{
  if (event == "INSERTED")
    event = "CREATED";
  if (event == "CHANGED")
    event = "UPDATED";
  if (event == "REMOVED")
    event = "DELETED";
  
  print class " " uid " " event;
}

BEGIN {
  isResultSeq         = 0;
  isInterpretationSeq = 0;
  isChanged           = 0;
  depth               = 0;
  currentResult           = "";
  currentInterpretation   = "";
}

#{
#  print $0;
#}


( $1 ~ "(f215,..31)") {
  if ($3 == "ENDSEQUENCE")
  {
#    print "END Results Seq";
    isResultSeq = 0;
  }
  else
  {
#    print "START Results Seq";
    depth = 0;
    isResultSeq = 1;
  }
  next;
}


( $1 ~ "(f215,..32)" ) {
  if ($3 == "ENDSEQUENCE")
  {
#    print "END Interpretation Seq";
    isInterpretationSeq = 0;
  }
  else
  {
#    print "START Interpretation Seq";
    depth = 0;
    isInterpretationSeq = 1;
  }
  next;
}


( $1 == "STARTITEM" ) {
  if (depth == 0)
  {
    if ( isResultSeq )
    {
#      print "START Results Item";
      currentResult = $3;
      result[ $3 ]  = $2;
      isChanged     = 0;
    }
  
    if (isInterpretationSeq)
    {
#      print "START Interpretation Item";
      currentInterpretation = $3;
      interpretation[ $3 ]  = $2;
      isChanged             = 0;
    }
  }

  depth++;
  next;
}

( $1 == "REMOVEITEM" ) {
  if ( isResultSeq )
    result[ $3 ]  = $2;
  if (isInterpretationSeq)
    interpretation[ $3 ]  = $2;
  
  next;
}

( $1 == "ENDITEM" ) {
  depth --;
  if (depth == 0)
  {
    if (isResultSeq)
    {
#      print "END Results Item";
      if (result[ currentResult ] == "CHANGED" && ! isChanged)
        result[ currentResult ] = "NONE";
      currentResult == "";
    }
    if (isInterpretationSeq)
    {
#      print "END Interpretation Item";
      if (interpretation[ currentInterpretation ] == "CHANGED" && ! isChanged)
        interpretation[ currentInterpretation ] = "NONE";
      currentInterpretation = "";
    }
  }
  next;
}


( $1 == "(4008,0212)" && (interpretation[currentInterpretation] != "INSERTED") ) {
  if (currentInterpretation != "" && $2 != "REMOVED" )
    interpretation[currentInterpretation] = $3;
  next;
}


{
  if (isResultSeq || isInterpretationSeq)
    isChanged = 1;
}



END {
  for ( val in result )
  {
    if ( val != "" && result[ val ] != "NONE" )
      printEventRow("results", val, result[val]);
  }

  for ( val in interpretation )
  {
    if ( val != "" && interpretation[ val ] != "NONE" )
      printEventRow("interpretation", val, interpretation[val]);
  }
}
