#!/bin/bash
#
# Component: dicom
#
# Description:
#    Gets orders from devices supporting Modality Worklist.
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="getorders.sh[$$]"

CWD=`pwd`

FROM_DAYS=""
TO_DAYS=""

RUNNING="$DICOM_VAR/mwl.running"

[ -f "$RUNNING" ] && exit
touch $RUNNING

SQL_RES=`echo "select HOST, PORT, AE, PROTOCOL from $TARGETTBL, $DEVSERVTBL where ID=DEVICE and SERVICE='MWLP';" |\
$MYSQL_HOME/bin/mysql -N $DBNAME`

if [ $? -ne 0 -o -z "$SQL_RES" ] ; then
  rm -f $RUNNING 2>/dev/null
  exit 1
fi

[ ! -d "$DICOM_VAR/orders" ] && mkdir $DICOM_VAR/orders
cd $DICOM_VAR/orders

if [ $? -ne 0 ] ; then
  rm -f $RUNNING 2>/dev/null
  exit 1
fi

[ -f "$DICOM_VAR/mwl.cfg" ] && . $DICOM_VAR/mwl.cfg

MWL_QUERY_FILE="$DICOM_HOME/fullquery.dcm"
[ -f "$DICOM_VAR/mwlquery.dcm" ] && MWL_QUERY_FILE="$DICOM_VAR/mwlquery.dcm"

[ -z "$FROM_DAYS" ] && FROM_DAYS=-1
[ -z "$TO_DAYS" ] && TO_DAYS=3

FROM_DATE=`date +%Y%m%d --date "$FROM_DAYS days"`
TO_DATE=`date +%Y%m%d --date "$TO_DAYS days"`

DATERANGE="$FROM_DATE-$TO_DATE"

TMP_QUERY_FILE=$TMP/mwlquery.$$.dcm
TMP_MOD_FILE=$TMP/mwlquery.$$.mod

cat << ENDOFMODFILE > $TMP_MOD_FILE
-t "(0040,0002)" -s "(0040,0100)" -r DA -v "$DATERANGE"
ENDOFMODFILE

$DCVIEWER_HOME/dcmod -f $TMP_MOD_FILE -o $TMP_QUERY_FILE $MWL_QUERY_FILE

if [ $? -ne 0 ] ; then
  rm $RUNNING
  rm $TMP_MOD_FILE
  exit 1
fi

[ -f "$DICOM_VAR/pb-scp.cfg" ] && . $DICOM_VAR/pb-scp.cfg
[ -z "$AE_TITLE" ] && AE_TITLE="$DICOM_AE_TITLE"
[ -n "$AE_TITLE" ] && OUR_AETITLE="-aet $AE_TITLE"

cat << ENDOFSQLRES | while read host port ae protocol; do
$SQL_RES
ENDOFSQLRES

  [ -z "$protocol" ] && continue;

  TLS_OPT=""
  [ "$protocol" = "S" ] && TLS_OPT="+tls $SSL_SITE_KEY $SSL_SITE_CERT -ic +ps"

  # run mwlfindscu
  if [ "$protocol" = "S" ] ; then
    $OPENSSL_PASS
  fi | \
  $DICOM_HOME/bin/mwlfindscu $TLS_OPT --call $ae $OUR_AETITLE $host $port $TMP_QUERY_FILE

  if [ $? -ne 0 ] ; then
    $LOGGER -p $LOG_FACILITY.err -t "$TAG" "Error getting orders from $host:$port:$ae"
  fi

done

cd $CWD

rm $TMP_QUERY_FILE
rm $TMP_MOD_FILE
rm $RUNNING
exit

