#!/bin/bash
#
## Component: dicom
##
## Description:
##	Canonize a DICOM object and send the normal form through standard output.
##
##	 Object's normal form is the herelt DICOM dump concatenated with all frames' raw.
##
## Usage:
##	objcanonize.sh [-i <ignoring_patterns_file>] <path_of_DICOM_File>
##
## Parameter(s):
##	-i <ignoring_patterns_file> : same as grep -vf <ignoring_patterns_file> but
##			default value : $ARCHIVE_HOME/ignoretags (when it's omitted)
##
## $Author$
## $Date$
##
#
#  RCS File Name    : $RCSfile$
#  RCS Revision     : $Revision$
#  Locked by        : $Locker$
#  Symbolic name    : $Name$
#  State            : $State$
#
. $HOME/etc/setenv.sh

usage="USAGE: objcanonize.sh [-i <ignoring_patterns_file>] <path_of_DICOM_File>"
if [ "--help" = "$1" ] ; then
	echo "$usage"
	exit 0
fi

DCMF="$1"
IGNORING_PATTERNS="$ARCHIVE_HOME/ignoretags"
if [ "-i" = "$1" ] ; then
	IGNORING_PATTERNS="$2"
	if [ ! -f "$IGNORING_PATTERNS" ] ; then
		echo "ERROR: <ignoring_patterns_file> parameter missed or $IGNORING_PATTERNS file does not exists." >&2
		echo "$usage" >&2
		exit 4
	fi
	DCMF="$3"
fi

if [ -z "$DCMF" -o ! -f "$DCMF" ] ; then
	echo "ERROR: <path_of_DICOM_File> parameter missed or $DCMF file does not exists." >&2
	echo "$usage" >&2
	exit 8
fi

DATE=`date +"%Y%m%d%H%M%S"`
WORK_DIR="$TMP/objnorm.$DATE.$$"
mkdir $WORK_DIR
if [ ! -d $WORK_DIR ] ; then
  echo "ERROR: working directory can not be created." >&2
  exit 16
fi

ERR=`$DCVIEWER_HOME/herelo -v -e $DCMF $WORK_DIR/obj 2>&1 >$WORK_DIR/dump`
if [ $? -ne 0 ] ; then
	echo "ERROR: can not herelo $DCMF:" >&2
	echo "$ERR" >&2
	rm -rf $WORK_DIR
	exit 125
fi

cat $WORK_DIR/dump | grep -vf "$IGNORING_PATTERNS" | sed 's;^\(([0-9a-f,]*)\) ?? ;\1 UN ;' >$WORK_DIR/obj
rm -f $WORK_DIR/dump

cat `ls $WORK_DIR/* | grep -v desc | grep -v 128x128 | sort -d`

rm -rf $WORK_DIR
