/**
 * Component: imgproc
 *
 * Description:
 *    Class for extracting document data from Encapsulated Document DICOM objects.
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */

#ifndef ENCAPSULATEDDOCUMENT_H_
#define ENCAPSULATEDDOCUMENT_H_

#include <dcmtk/ofstd/ofstring.h>

#include "imgproc/extracteddata.h"

/**
 * Supported document types
 */
typedef enum {
		/** Unknown/unrecognized document */
		DOC_TYPE_UNKNOWN,
		/** PDF documents */
		DOC_TYPE_PDF
} DOCUMENT_TYPE;

/**
 * Base class for extracting document from Encapsulated Document DICOM objects.
 */
class CEncapsulatedDocument : public CExtractedData {
	private:

		/** Document type */
		DOCUMENT_TYPE m_documentType;

	public:
		/**
		 * Constructor for creating a new Encapsulated Document.
		 *
		 * @param imageProcess   Image Process object holding original DICOM data
		 * @param type           Document type (Currently the only supported type is PDF)
		 * @param length         Length of the Document
		 */
		CEncapsulatedDocument( CImageProcess * imageProcess, DOCUMENT_TYPE type, Uint32 length );

		/**
		 * Destructor for destroying this encapsulated document
		 */
		virtual ~CEncapsulatedDocument();


		/**
		 * Add contents (document data) to this encapsulated document object
		 *
		 * @param content   document content to be added
		 * @param length    length of the document content (should be the same as provided in the constructor)
		 */
		void addContent( Uint8 * content, Uint32 length );

		/**
		 * Initializes CEncapsulatedDocument object for processing.
		 *
		 * @return OFTrue on success; OFFalse on error
		 */
		virtual OFBool init();

		/**
		 * Post processes encapsulated document
		 *
		 * @return 0 on success; >0 on error
		 */
		virtual int postProcess();

		/**
		 * @return  extension based on document type
		 */
		OFString getExtension() {
			return m_documentType == DOC_TYPE_PDF ? "pdf" : "";
		}
};

#endif /* ENCAPSULATEDDOCUMENT_H_ */
