/*
 * Component:  imutils
 *
 * Purpose:
 *    Transaction manager (handling transactions by a unique Transaction ID)
 *
 */
#ifndef IMTRANSACTIONMANAGER_H_INCLUDED
#define IMTRANSACTIONMANAGER_H_INCLUDED

#include <stdarg.h>
#include <string>

#include <imutils/DBConnector.h>

#define TRANSACTION_STATE_INPROCESS "InProcess"
#define TRANSACTION_STATE_COMPLETED "Completed"


namespace erad::imutils {


class IMTransactionManager {
	private:

		/// database connection
		erad::db::DBConnector& db;
		/// transaction identifier
		std::string m_transID;

	public:

		IMTransactionManager( erad::db::DBConnector& db, const std::string& transID );

		/**
		 * Returns if for the given transaction ID a valid answer (result) is found from a previous operation
		 *   (this should be returned back to the caller without doing the operation again)
		 * Returns NULL in all other cases (no database, no answer, transaction is InProcess, ...)
		 *
		 * @return if for the given transaction ID a valid answer (result) is found from a previous operation
		 */
		bool getAnswer( std::string& answer /* out */);

		/**
		 * Sets the answer (result) for the given transaction ID, also sets its state to Completed
		 * 	Should be used only if the operation was successful
		 *
		 * @param answer the result of the operation
		 * @return if succeeded
		 */
		bool setAnswer( const std::string& answer );

		/**
		 * Returns if the given transaction is found and it's in InProcess state
		 *
		 * @return if the given transaction is found and it's in InProcess state
		 */
		bool isInprocess();

		/**
		 * Sets the state of the given transaction to InProcess
		 * 	Should be used at the beginning of a transaction when it takes formidable time to execute it
		 *    setAnswer Should be called when the transaction is ready
		 *
		 * @return
		 */
		bool setInprocess();

};

} // namespace

#endif
