#ifndef ERAD_IMUTILS_SELFREC_H_
#define ERAD_IMUTILS_SELFREC_H_

#include <memory>
#include "Config.h"

namespace erad {
namespace imutils {

/**
 * Helper class to read timeout configuration from config file.
 *
 * Reads the value of SQLTimeLimit setting in $CONF/self.rec
 *
 * By default it checks the config for changes after 1 seconds since last check.
 *
 * Not thread safe
 *
 * Typical usage:
 *
 * SelfRec::Instance().getter_method
 */
class SelfRec : Config {
	private:
		unsigned long int queryTimeout;

		/** ProcessMode = database*/
		bool cwl;

		bool objectTable;

		bool origin;

		bool replica;

		static std::unique_ptr< SelfRec > INSTANCE;

		// default constructor
		SelfRec();

		// copy constructor - disabled
		SelfRec( const SelfRec& ) = delete;
		// move constructor - disabled
		SelfRec( SelfRec&& ) = delete;
		// copy assignment - disabled
		SelfRec& operator=( const SelfRec& ) = delete;
		// move assignment - disabled
		SelfRec& operator=( SelfRec&& ) = delete;

	protected:
		virtual void load();
		virtual void reset();

	public:
		virtual ~SelfRec();

		unsigned long int getQueryTimeout() const {
			return queryTimeout;
		}

		bool isCwl() const {
			return cwl;
		}

		bool isObjectlessCwl() const {
			return cwl && !objectTable;
		}

		/**
		 * @return true if and only if this server is an origin (master of dicom data)
		 */
		bool isOrigin() const {
			return origin;
		}

		/**
		 * generic access to self.rec properties
		 * @param name property name
		 * @return value of the property or NULL
		 */
		inline const char* getProperty( const char *name ) const {
			return configPrp.getProperty( name );
		}

		/**
		 * @return true if and only if this server is a replica (slave of dicom data)
		 */
		bool isReplica() const {
			return replica;
		}

		static const SelfRec& Instance();
};

}
} // namespace

#endif /* ERAD_IMUTILS_SELFREC_H_ */
