/*
 * Component: imutils
 *
 * Description:
 *     Reading text from a socket, buffering characters to provide efficient reading lines.
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */
#ifndef __IMUTILS_SOCKET_READER_H_INCLUDED__
#define __IMUTILS_SOCKET_READER_H_INCLUDED__

#include <string>

/**
 * Reading text from a socket, buffering characters to provide efficient reading lines.
 */
class SocketReader {
private:

	/** Socket to read from */
	int m_socket;

	/** Buffer to read into */
	char * m_buffer;

	/** Size of the buffer */
	int m_bufferSize;

	/** Current position in the buffer (cursor position) */
	int m_cur;

	/** Position of the last char in the buffer (actual size of the buffer content - 1) */
	int m_last;

	/** Is EOF reached when reading from the socket */
	bool m_isEOF;

	/** Fills the buffer by reading content from socket */
	void fillBuffer();

	/*
	 * Print debug messages with buffer status to standard output
	 *
	 * @param msg   Message to be printed before the buffer status
	 */
//	void printDebug(const char * msg);

public:

	/**
	 * Constructor for creating a new SocketReader object.
	 *
	 * @param socket       The socket to read
	 * @param bufferSize   Input buffer size
	 */
	SocketReader( int socket, int bufferSize );

	/**
	 * Destructor for destroying the SocketReader object.
	 */
	~SocketReader();


	/**
	 * Reads characters from the socket upto the first new line character ('\n') to line.
	 * The new line character will not be returned
	 *
	 * @param line    output parameter for the line read from the socket
	 *
	 * @return true if a line has successfully read, false on error or end of characters on the socket (closed)
	 */
	bool readLine( std::string & line );

	/**
	 * @return true if and only if there is no more character available on the socket (closed)
	 */
	inline bool isEOF() {
		return m_isEOF;
	}
};

#endif // __IMUTILS_SOCKET_READER_H_INCLUDED__
