/**
 * Component: 
 *
 * Description:
 * 
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 */

#ifndef IMDATE_H_
#define IMDATE_H_

#include <string>

using namespace std;

/**
 * Wrapper class to represent a calendar date.
 */
class IMDate {
	private:

		/** Array of days in months */
		static int DAYS[];

		/** years */
		int m_year;
		/** month in year 1-12 */
		int m_month;
		/** day in month 1-31 */
		int m_day;

		/** Indicates whether this date is valid or not */
		bool m_isValid;

		/**
		 * Checks if the date to be represented is valid.
		 * Sets isValid flag.
		 */
		void check();

		/**
		 * Checks if the provided string consist of digits only
		 *
		 * @param str  string to check (should not be NULL)
		 * @return true if and only if the string consist of digits only
		 */
		bool isDigit( const char * str );

		/**
		 * Check if year is a leap year:
		 *     * the year is divisible by 4 but not divisible by 100, or
		 *     * the year is divisible by 400
		 * @param year   year to check
		 * @return  true if and only if the year is a leap year
		 */
		static bool staticIsLeapYear( int year );

		/**
		 * @return the number of days in the given month of the given year.
		 */
		static int staticGetDaysInMonth( int year, int month );

		/**
		 * Check if year is a leap year:
		 *     * the year is divisible by 4 but not divisible by 100, or
		 *     * the year is divisible by 400
		 *
		 * @return  true if and only if this year is a leap year
		 */
		inline bool isLeapYear() const {
			return staticIsLeapYear( m_year );
		}

		/**
		 * @return the number of days in the current month of the current year.
		 */
		inline int getDaysInMonth() const {
			if ( m_month != 2 ) {
				return DAYS[ m_month - 1 ];
			}

			// check if leap year
			return isLeapYear() ? 29 : 28;
		}

		/**
		 * @return number of days within the year for the date represented by this object
		 */
		inline int dayInYear() const {
			int d = 0;
			for ( int i = 0; i < m_month; i++ ) {
				d += staticGetDaysInMonth( m_year, i );
			}

			return d + m_day;
		}

	public:

		/**
		 * Constructor for creating a new Date object
		 *
		 * @param dateStr   Date to be represented (8 digit: YYYYMMDD)
		 */
		IMDate( const char * dateStr );

		/**
		 * @return  true if and only if this date is valid
		 */
		inline bool isValid() const {
			return m_isValid;
		}

		/**
		 * Checks if this date is less then the argument date.
		 *
		 * @param date
		 * @return
		 */
		inline bool operator<( const IMDate & date ) const {
			if ( m_year == date.m_year ) {
				if ( m_month == date.m_month ) {
					return m_day < date.m_day;
				}
				return m_month < date.m_month;
			}

			return m_year < date.m_year;
		}

		/**
		 * Calculates age elapsed since another date.
		 *   - if age is larger than or equals 1 year then age will be returned in years
		 *   - else if age is larger than or equals 1 month then age will be returned in months
		 *   - otherwise age will be returned in days
		 *
		 * @param date  calculates age since this date
		 * @return age in DICOM conformant format or empty string if either this or argument date is invalid or this date predates argument date.
		 */
		string ageSince( const IMDate & date );

		/**
		 * @return the actual date/time in YYYYMMDD_HHMMSS format
		 */
		static string staticGetActualDateTime();
};

#endif /* IMDATE_H_ */
