#ifndef CLOUDACCESS_H
#define CLOUDACCESS_H

#include <string>

#include "utils/CloudDicomStore.h"

/**
 * This is the class used to get an access to the cloud storage.
 * It create and returns cloud store object
 */
class CloudAccess {
		/*
		 * To keep Project name details
		 */
		std::string m_HealthcareProject;
		/*
		 * To keep region name details
		 */
		std::string m_HealthcareRegion;
		/*
		 * To keep Dataset name details
		 */
		std::string m_HealthcareDataset;
		/*
		 * To keep dicom store name details
		 */
		std::string m_HealthcareDicomId;
		/*
		 * Tocken details
		 */
		std::string m_Token;

	public:

		/**
		 * Constructor for CloudAccess
		 * @param studyUID - String buffer to read from
		 * @return nil
		 * @exception - std::runtime_error will be thrown when token generation failed.
		 */
		CloudAccess();
		/**
		 * Destructor for CloudAccess
		 */
		virtual ~CloudAccess();

		/**
		 * Function to get the cloud dicom store details
		 */
		std::string getCloudDicomStorePath( const std::string& studyUID );

		/**
		 * Function to get the Dicom store object
		 */
		std::unique_ptr<CloudDicomStore> getDicomStoreForStudy( const std::string& studyUID );

		/**
		 * Function to get the Dicom store object from dicom store path
		 */
		std::unique_ptr<CloudDicomStore> getDicomStoreFromPath( const std::string& dicomStorePath );
};

#endif // CLOUDACCESS_H
