#ifndef CLOUDDICOMHELPER_H
#define CLOUDDICOMHELPER_H

#include <string>
#include <utils/CloudDicomStore.h>

/**
 * Helper class to store Local DICOM File path and perform cleanup
 */
class LocalDicomFile {
		std::string m_absolutePath;

	public:
		LocalDicomFile( const std::string& absolutePath );
		~LocalDicomFile();

		// Delete copy constructor and assignment operator to prevent copying
		LocalDicomFile() = delete;
		LocalDicomFile( const LocalDicomFile& ) = delete;
		LocalDicomFile( const LocalDicomFile&& ) = delete;
		LocalDicomFile& operator=( const LocalDicomFile& ) = delete;
		LocalDicomFile& operator=( const LocalDicomFile&& ) = delete;

		inline const std::string& getLocalDicomFilePath() const {
			return m_absolutePath;
		}

};

/**
 * Helper class to call functions of CloudDicomStore
 */
class CloudDicomHelper {
		CloudDicomHelper();

		// Delete copy constructor and assignment operator to prevent copying
		CloudDicomHelper( const CloudDicomHelper& ) = delete;
		CloudDicomHelper( const CloudDicomHelper&& ) = delete;
		CloudDicomHelper& operator=( const CloudDicomHelper& ) = delete;
		CloudDicomHelper& operator=( const CloudDicomHelper&& ) = delete;
	public:

		static CloudDicomHelper& GetInstance() {
			static CloudDicomHelper instance;
			return instance;
		}

		std::unique_ptr<LocalDicomFile> retrieveFileFromSourceDevice( std::unique_ptr<CloudDicomStore> dicomStore, const std::string& basePath,
				const std::string& relativeFile, bool isPbrFile = false,
				const std::string& objectFile = "" );
};

#endif // CLOUDDICOMHELPER_H
