#ifndef HTTP_H
#define HTTP_H

#include "HttpClientT.h"

#include <boost/asio/ssl.hpp>


class HttpConnector {
protected:
    static const char* s_Service;

    using Stream = boost::beast::tcp_stream;

protected:
    std::chrono::milliseconds m_ConnectTimeout;

    Stream connect(boost::asio::io_context& ioContext, const HttpEndpoint& endpoint);
    void close(Stream& stream);

public:
    HttpConnector();

    void setConnectTimeout(const std::chrono::milliseconds& timeout) {
        m_ConnectTimeout = timeout;
    }
};


class HttpsConnector {
protected:
    static const char* s_Service;

    using Stream = boost::asio::ssl::stream<boost::beast::tcp_stream>;

protected:
    boost::asio::ssl::context m_SSLContext;
    std::chrono::milliseconds m_ConnectTimeout;
    std::chrono::milliseconds m_HandshakeTimeout;

    Stream connect(boost::asio::io_context& ioContext, const HttpEndpoint& endpoint);
    void close(Stream& stream);

public:
    HttpsConnector();

    void setConnectTimeout(const std::chrono::milliseconds& timeout) {
        m_ConnectTimeout = timeout;
    }

    void setHandshakeTimeout(const std::chrono::milliseconds& timeout) {
        m_HandshakeTimeout = timeout;
    }

    void disableCertificateValidation();
};


using HttpClient = HttpClientT<HttpConnector>;
using HttpsClient = HttpClientT<HttpsConnector>;

#endif // HTTP_H