/*
 * Component: utils
 *
 * Purpose:
 *    Header file for config file reader implemenatation.
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$
 * State            : $State$
 *
 */
#ifndef __UTILS_IMCONFIG_H_INCLUDED__
#define __UTILS_IMCONFIG_H_INCLUDED__

#include <string.h>

class IMConfig;

/**
 * Entry in a config file
 */
class IMConfigEntry {
		friend class IMConfig;

	protected:

		/** Identifies the entry in the config file */
		const char * m_id;

		virtual void init();
		char * checkAndCopy( const char * val );
		char * readNextItem( char *& cur );

	public:

		inline const char * getID() const {
			return m_id;
		}

		// List

	private:

		bool m_isHeader;
		IMConfigEntry * m_next;

	protected:

		IMConfigEntry();
		IMConfigEntry( const char * id );
		virtual ~IMConfigEntry();

		inline void setNext( IMConfigEntry * next ) {
			m_next = next;
		}
		inline IMConfigEntry * getNext() {
			return m_next;
		}
		inline void setID( const char * id ) {
			if ( m_isHeader )
				m_id = id;
		}
		inline bool isIDEquals( const char * id ) {
			return ( id == m_id ) || ( id != NULL && m_id != NULL && strcmp( m_id, id ) == 0 );
		}
};

class IMConfig {
	private:
		IMConfigEntry * m_head;
		time_t m_lastModified;
		char * m_configFileName;

		virtual IMConfigEntry * processLine( char * line ) = 0;
		virtual IMConfigEntry * createHeadEntry() = 0;
		void clearEntryList( IMConfigEntry * head );

	protected:
		char * readNextItem( char *& cur );

	public:
		IMConfig( const char * configFileName );
		IMConfig();
		virtual ~IMConfig();

		bool checkAndReadConfigFile();
		bool checkAndReadConfigFile( const char * configFileName );

		IMConfigEntry * getEntryFor( const char * id );

		IMConfigEntry * getFirstEntry() {
			return ( m_head == NULL || m_head->getNext() == m_head ) ? NULL : m_head->getNext();
		}
		IMConfigEntry * getNextEntry( IMConfigEntry * entry ) {
			return ( entry == NULL || entry->getNext() == m_head ) ? NULL : entry->getNext();
		}
};

#endif // __UTILS_IMCONFIG_H_INCLUDED__
