#ifndef UTILS_H
#define UTILS_H

#include <map>
#include <sstream>
#include <string>


template <class t_iter, class t_stringify>
std::string join(const t_iter& itBegin, const t_iter& itEnd, const char* szDelim, const t_stringify& stringify)
{
    std::ostringstream ss;

    if (itBegin != itEnd) {
        t_iter it = itBegin;

        ss << stringify(*it);
        ++it;

        for ( ; it != itEnd; ++it) {
            ss << szDelim << stringify(*it);
        }
    }
    
    return ss.str();
}

template <class t_iter>
std::string join(const t_iter& itBegin, const t_iter& itEnd, const char* szDelim)
{
    return join(itBegin, itEnd, szDelim, [] (const typename t_iter::reference i) { return i; });
}

template <class t_key, class t_value>
bool mapKeyExists(const std::map<t_key, t_value>& map, const t_key& key)
{
    auto it = map.find(key);

    return it != map.end();
}

template <class t_key, class t_value>
t_value mapGetOrDefault(const std::map<t_key, t_value>& map, const t_key& key, const t_value& defaultValue)
{
    auto it = map.find(key);
    if (it != map.end())
        return it->second;
    else
        return defaultValue;
}

template <class T>
T fromString(const std::string& str, const T& defaultValue)
{
    std::istringstream iss(str);
    T value;

    if (!(iss >> value))
        return defaultValue;
    else
        return value;
}

std::string tolower(std::string str);

#endif // UTILS_H