#!/bin/bash
#
# Checks if upgrade to v7.3 can convert the database tables.  
#  - checks duplicate study table entries (copies in a folder using the original study UID)
#

. "$HOME/etc/setenv.sh"

studiesFile="$TMP/check73upgrade.studies"

tempFile="${studiesFile}.$$"

sql.sh "select count(*) as N, STYIUID from ${STUDYTBL} group by STYIUID having N>1" -N | sed "s;^[0-9]*[[:space:]]*;;" > "${tempFile}"


if [ -s "${tempFile}" ] ; then

	#	numStudies=$( wc -l "${tempFile}" | awk '{print $1}'
	mv "${tempFile}" "${studiesFile}"
			
	cat <<-EOFERR
		Database entries for some studies need to be cleaned up prior to upgrade to v7.3.
		
		Find the list of studies to check and cleanup in ${studiesFile}.
		It's recommended to run checkintegrity on these studies:

		        $CHECKINTEGRITY_HOME/checkAll.sh -f ${studiesFile}

	EOFERR

	exit 1
fi
		
echo "OK. No cleanup needed."

exit 0
