#!/bin/bash
#
# Component: tools
#
# Description:
#		Compares results of getstates.sh. Will compare the study lists of a child server
# created on the child and on the parent server.
#
# Usage:
#		diffstates <parent list> <child list> <outprefix>
#
# diffstates will generate output files with <outprefix>
#
#   <outprefix>.OK         : These studies exist on both the child and the parent and are consistent.
#   <outprefix>.DIFFER     : These studies exist on both the child and the parent but are inconsistent.
#   <outprefix>.onlyChild  : These studies exist only on the child server (also contains the Owner/Staged flags)
#   <outprefix>.onlyParent : These studies exist only on the parent server (also contains the Owner/Staged flags)
#
#  Last Modified by : $Author$
#  Last Modified at : $Date$
#
#  RCS File Name    : $RCSfile$
#  RCS Revision     : $Revision$
#  Locked by        : $Locker$
#  Symbolic name    : $Name$
#  State            : $State$
#


PARENTLS=$1
CHILDLS=$2
PREFIX=$3

if [ -z "$PREFIX" ] ; then
  echo "usage:"
  echo "   diffstates.sh <parent list> <child list> <outprefix>"
fi

[ ! -f "$PARENTLS" -o ! -f "$CHILDLS" ] && exit 1


MERGELS="$PREFIX.merge"
DIFFLS="$PREFIX.res"

sort -m $CHILDLS $PARENTLS > $MERGELS

[ ! -f "$MERGELS" ] && exit 1

awk ' BEGIN { s="-"; o="-"; printNext = 0; } \
     (NF != 0 && $1 == s) { printNext = 0; if ($2 == o) { print "OK " $1; } else { print "DIFFER " $1; } } \
     (NF != 0 && $1 != s) { if (printNext == 1) { print "SOLO " s; } s = $1; o = $2; printNext = 1; }  \
     END { if (printNext == 1) { print "SOLO " s; } }' $MERGELS > $DIFFLS

grep "^OK" $DIFFLS | awk '{print $2}' > $PREFIX.OK
grep "^DIFFER" $DIFFLS | awk '{print $2}' > $PREFIX.DIFFER
grep "^SOLO" $DIFFLS | awk '{print $2}' > $PREFIX.SOLO

grep -f $PREFIX.SOLO $CHILDLS > $PREFIX.onlyChild
grep -f $PREFIX.SOLO $PARENTLS > $PREFIX.onlyParent

rm $PREFIX.merge $PREFIX.SOLO $PREFIX.res 2>/dev/null
