<%--
 * Component: tools
 *
 * Description:
 *   storestate checker
 *
 * Parameters:
 *
 *  cDetailed: if "true", it will result a detailed output (result will be no "OK" or "failed", but
 *              lists describing the status/checksum of studies and their objects on this server)
 *
 *  cStudies:  list of studies to check
 *	cAll:			check all studies if it's value is "true"
 *
 *	cFromRoot:	proxy to dotcom root and check there if it's value is "true"
 *
 *  Last Updated     : $Author$
 *  Update Date Time : $Date$
 *  RCS File Name    : $RCSfile$
 *  RCS Revision     : $Revision$
 *  Locked by        : $Locker$
 *  Symbolic name    : $Name$
 *  State            : $State$
--%><%@
	page import="imagemed.jsputil.*, imagemed.imsession.*, imagemed.util.*, java.sql.*, java.util.*, imagemed.http.*, java.io.*, java.net.*"

%><%@
	page errorPage="../main/exceptionHandler.jsp"
	
%><%!protected static final boolean DEBUG = false;
	protected static Syslog m_logger;

	static {
		m_logger = new Syslog( "checkStudies.jsp", 0, IMEnvBase.getEnv( "LOG_FACILITY" ) );
	}

	protected void debug( String msg ) {
		if ( DEBUG ) {
			m_logger.syslog( Syslog.LOG_DEBUG, msg );
		}
	}

	//------------------------------------------------------------------------------------------------

	static final String COUNTER = "counter describing number of running child-processes";
	static final String CHILDERRORS = "error reporting facility for child processes";
	static final String STUDIESPAR = "studies to be checked";
	static final String SMD5SMAP = "data structure containing MD5 checksums for objects of studies";
	static final String ISLOCALREADY = "trigger like thing to signal child processes when local MD5 checksums are generated";
	static final String TEMPDIR = "temp directory to store temp files...";

	//------------------------------------------------------------------------------------------------

	class Boolean2 {

		private boolean m_value;

		public Boolean2( boolean value ) {
			m_value = value;
		}

		public boolean booleanValue() {
			return m_value;
		}

		public void negate() {
			m_value = !m_value;
		}
	}

	//------------------------------------------------------------------------------------------------

	class ChildChecker implements Runnable {

		private final String m_childID;
		private final Counter counter;
		private final SynchronizedProperties errorMessages;
		private final String studies;
		private final StudyMD5sMap sm5Map;
		private final Boolean2 isLocalReady;
		private final String tempDir;
		private final boolean isSingleStudy;

		public ChildChecker( String childID, Map<String, Object> environment ) {
			m_childID = childID;
			counter = (Counter) environment.get( COUNTER );
			errorMessages = (SynchronizedProperties) environment.get( CHILDERRORS );
			studies = (String) environment.get( STUDIESPAR );
			sm5Map = (StudyMD5sMap) environment.get( SMD5SMAP );
			isLocalReady = (Boolean2) environment.get( ISLOCALREADY );
			tempDir = (String) environment.get( TEMPDIR );
			isSingleStudy = !( "&cAll=true".equals( studies ) || studies.indexOf( ";" ) >= 0 );
		}

		private final void addError( String error ) {
			String errMsg = errorMessages.getProperty( m_childID );
			if ( errMsg == null ) {
				errMsg = "";
			}
			else if ( !errMsg.endsWith( "\n" ) ) {
				errMsg += "\n";
			}
			errMsg += error;
			errorMessages.setProperty( m_childID, errMsg );
		}

		public String callChild( String childHost ) {
			StringBuffer result = new StringBuffer();
			try {
				URL thepage = new URL( "https://" + childHost + "/tools/checkStudies.jsp?cDetailed=true" + studies );
				HttpURLConnection conn = (HttpURLConnection) IMProxy.openConnection( thepage, (String) null, null );

				InputStream inStream = conn.getInputStream();

				byte[] buffer = new byte[ 1024 ];
				int nReadBytes = 0;

				while ( ( nReadBytes = inStream.read( buffer ) ) > 0 ) {
					result.append( new String( buffer, 0, nReadBytes ) );
				}

				inStream.close();
				conn.disconnect();
			}
			catch ( Throwable th ) {
				result.append( "#ERROR: Connection problem: " + th.toString() );
			}
			return result.toString();
		}

	   @SuppressWarnings( "all" )
		public void run() {
			try {
				counter.increaseCounter();
				String childHost = "";
				String thisHost = IMEnvBase.getEnv( "SERVERNAME" );
				try {
					ResultSet rs = IMEnv.getDB().createStatement().executeQuery( "select HOST from " +
							IMEnvBase.getEnv( "TARGETTBL" ) + " where ID='" + m_childID + "'" );
					if ( rs.next() ) {
						childHost = rs.getString( "HOST" );
					}
					else {
						addError( "Unknown childID!" );
						counter.decreaseCounter();
						return;
					}
				}
				catch ( SQLException se ) {
					addError( "Database failed during child host identification!" );
					counter.decreaseCounter();
					return;
				}
				String output = callChild( childHost );
				while ( !isLocalReady.booleanValue() ) {
					Thread.yield();
				}
				if ( output.startsWith( "#ERROR:\n" ) ) {
					int p = output.indexOf( "#RESULT:\n" );
					if ( p < 0 ) {
						p = output.length();
					}
					String errMsg = output.substring( 8, p ).trim();
					addError( errMsg );
					try {
						output = output.substring( p );
					}
					catch ( IndexOutOfBoundsException ioobe ) {
						output = "";
					}
				}
				if ( output.startsWith( "#RESULT:\n" ) ) {
					output = output.substring( 8 ).trim();
					String md5s = "";
					int p = output.indexOf( "#MD5s:" );
					if ( p >= 0 ) {
						md5s = output.substring( p + 6 ).trim();
						output = output.substring( 0, p );
					}
					Helper.writeTextFile( tempDir + "/child." + m_childID + ".far.ls", output );
					String[] cmd = new String[ 4 ];
					cmd[ 0 ] = IMEnvBase.getEnv( "TOOLS_HOME" ) + "/diffstates.sh";
					cmd[ 1 ] = tempDir + "/child." + m_childID + ".ls";
					cmd[ 2 ] = tempDir + "/child." + m_childID + ".far.ls";
					cmd[ 3 ] = tempDir + "/child." + m_childID;
					RunCommandResult rcr = Helper.runCommandEx( cmd, new String[ 0 ] );
					if ( rcr.getExitCode() != 0 ) {
						addError( "diffstates.sh failed! " + rcr.getError() + " | " + rcr.getOutput() );
					}
					else {
						String differs = Helper.readTextFile( tempDir + "/child." + m_childID + ".DIFFER" );
						if ( differs != null && differs.length() > 0 ) {
							if ( isSingleStudy ) {
								addError( "The study has ownership mismatch between " + thisHost + " (parent) and " + childHost + " (child)." );
							}
							else {
								String ending = ( differs.trim().indexOf( "\n" ) < 0 ) ? "y" : "ies";

								addError( "The following stud" + ending + " has ownership mismatch between " + thisHost + " (parent) and " + childHost + " (child).\n" + differs );
							}
						}
						String parentOnly = Helper.readTextFile( tempDir + "/child." + m_childID + ".onlyParent" );
						if ( parentOnly != null && parentOnly.length() > 0 ) {
							if ( isSingleStudy ) {
								addError( "The study has a reference on " + thisHost + " (parent) to " + childHost + " (child), but doesn't exist there." );
							}
							else {
								String ending = ( parentOnly.trim().indexOf( "\n" ) < 0 ) ? "y" : "ies";

								addError( "The following stud" + ending + " have references on " + thisHost + " (parent) to " + childHost + " (child), but don't exist there: \n" + parentOnly );
							}
						}
						String childOnly = Helper.readTextFile( tempDir + "/child." + m_childID + ".onlyChild" );
						if ( childOnly != null && childOnly.length() > 0 ) {
							if ( isSingleStudy ) {
								addError( "The study exist on " + childHost + " (child), but have no reference on " + thisHost + " (parent)." );
							}
							else {
								String ending = ( childOnly.trim().indexOf( "\n" ) < 0 ) ? "y" : "ies";

								addError( "The following stud" + ending + " exist on " + childHost + " (child), but have no reference on " + thisHost + " (parent): \n" + childOnly );
							}
						}
					}
					if ( md5s.length() > 0 ) {
						String err = sm5Map.parseOutput( md5s, m_childID );
						if ( err != null && err.length() > 0 ) {
							addError( err );
						}
					}
				}
				else {
					addError( "Host did not return valid result: \n    " + Helper.replace( output, "\n", "\n    " ) );
					output = "";
				}
			}
			catch ( Throwable th ) {
				addError( "ChildCheck throwed: " + th.getClass() + ": " + th.getMessage() );
			}
			counter.decreaseCounter();
		}

	}

	//---------------------------------------------------------------------------------------------------

	class StudyMD5sMap {

		private Map<String, StudyMD5s> map = new HashMap<String, StudyMD5s>();

		public synchronized StudyMD5s getStudyMD5s( String study ) {
			StudyMD5s studyMD5s = map.get( study );
			if ( studyMD5s == null ) {
				studyMD5s = new StudyMD5s();
				map.put( study, studyMD5s );
			}
			return studyMD5s;
		}

		public boolean contains( String study ) {
			return map.containsKey( study );
		}

		public String getOutput() {
			String[] studies = map.keySet().toArray( new String[ 0 ] );
			String result = "";
			for ( int i = 0; i < studies.length; i++ ) {
				result += " Study: " + studies[ i ] + "\n";
				StudyMD5s smd5s = map.get( studies[ i ] );
				result += "  " + Helper.replace( smd5s.getOutput(), "\n", "\n  " );
				if ( !result.endsWith( "\n" ) ) {
					result += "\n";
				}
			}
			if ( result.endsWith( "\n" ) ) {
				result = result.substring( 0, result.length() - 1 );
			}
			return result;
		}

		public synchronized String parseOutput( String output, String hostID ) {
			StringTokenizer st = new StringTokenizer( output, "\n" );
			if ( !st.hasMoreTokens() ) {
				return "";
			}
			String result = "";
			String line = st.nextToken();
			while ( st.hasMoreTokens() ) {
				line = line.trim();
				if ( !line.startsWith( "Study: " ) ) {
					result += "Malformed output for MD5 section!";
					break;
				}
				line = line.substring( 6 ).trim();
				StudyMD5s sm = getStudyMD5s( line );
				String smProps = "";
				while ( st.hasMoreTokens() ) {
					line = st.nextToken();
					if ( !line.startsWith( "Study: " ) ) {
						smProps += line.trim() + "\n";
					}
					else {
						break;
					}
				}
				result += sm.parseOutput( smProps, hostID );
				if ( result.length() > 0 && !result.endsWith( "\n" ) ) {
					result += "\n";
				}
			}
			return result;
		}

		public String getErrors() {
			String result = "";
			Iterator<String> studyIterator = map.keySet().iterator();
			while ( studyIterator.hasNext() ) {
				String study = studyIterator.next();
				String err = map.get( study ).getErrors();
				if ( err.length() > 0 ) {
					result += " Study " + study + ":\n";
					result += err;
				}
			}
			return result;
		}

	}

	//---------------------------------------------------------------------------------------------------

	class StudyMD5s extends SynchronizedProperties {

		Map<String, Set<String>> objHosts = new HashMap<String, Set<String>>();

		public synchronized String setProperty( String objectFileName, String value, String hostID ) {
			String result = "";
			Set<String> objSet = objHosts.get( objectFileName );
			String oldValue = getProperty( objectFileName );
			if ( objSet == null ) {
				objSet = new HashSet<String>();
				objHosts.put( objectFileName, objSet );
				setProperty( objectFileName, value );
			}
			else if ( !oldValue.equals( value ) ) {
				result += "Object " + objectFileName + " doesn't match on hosts " + hostID + " and " + objSet.iterator().next() + "!\n";
			}
			objSet.add( hostID );
			return result;
		}

		public String getOutput() {
			return getString();
		}

		public String parseOutput( String output, String hostID ) {
			SynchronizedProperties prp = new SynchronizedProperties();
			prp.parseString( output );
			Enumeration<?> keys = prp.keys();
			String result = "";
			while ( keys.hasMoreElements() ) {
				String objectFileName = (String) keys.nextElement();
				String md5 = prp.getProperty( objectFileName );
				result += setProperty( objectFileName, md5, hostID );
				if ( result.length() > 0 && !result.endsWith( "\n" ) ) {
					result += "\n";
				}
			}
			return result;
		}

		public String getErrors() {
			if ( objHosts.isEmpty() ) {
				return "";
			}
			String result = "";
			Iterator<Set<String>> sets = objHosts.values().iterator();
			Set<String> allHosts = new HashSet<String>();
			while ( sets.hasNext() ) {
				allHosts.addAll( sets.next() );
			}
			Iterator<String> objFNs = objHosts.keySet().iterator();
			while ( objFNs.hasNext() ) {
				String objFN = objFNs.next();
				Set<String> hostIDs = objHosts.get( objFN );
				if ( hostIDs.size() != allHosts.size() ) {
					String badHosts = "";
					Iterator<String> hostIterator = allHosts.iterator();
					while ( hostIterator.hasNext() ) {
						String nextHost = hostIterator.next();
						if ( !hostIDs.contains( nextHost ) ) {
							badHosts += ", " + nextHost;
						}
					}
					badHosts = badHosts.substring( 2 ); // cutting trailing ", "
					result += "\n  Object " + objFN + " is missing on the following host(s): " + badHosts;
				}
			}
			if ( result.length() > 0 ) {
				result = result.substring( 1 ); // cutting trailing "\n"
			}
			return result;
		}

	}

	//-----------------------------------------------------------------------------------------------------

	private String addError( String errors, String error ) {
		if ( error == null || error.length() == 0 ) {
			return errors;
		}
		if ( errors.length() > 0 && !errors.endsWith( "\n" ) ) {
			errors += "\n";
		}
		errors += error;
		return errors;
	}

	private void generateOutput( IMSession is, boolean detailed, StudyMD5sMap smd5sMap, SynchronizedProperties childErrors, String errors, String tempDir ) throws IOException {
		String resultTag = "OK";
		String errorLog = Helper.readTextFile( tempDir + "/error.log" );
		if ( errorLog == null ) {
			errorLog = "";
		}
		if ( errorLog.length() > 0 ) {
			errors = addError( errors, "error.log:\n " + Helper.replace( errorLog, "\n", "\n " ) );
		}
		if ( !childErrors.isEmpty() ) {
			Enumeration<?> keys = childErrors.keys();
			while ( keys.hasMoreElements() ) {
				String key = (String) keys.nextElement();
				errors = addError( errors, "while comparing with " + key + ":" );
				String childError = childErrors.getProperty( key );
				if ( childError.endsWith( "\n" ) ) {
					childError = childError.substring( 0, childError.length() - 1 );
				}
				errors = addError( errors, "* " + Helper.replace( childError, "\n", "\n* " ) );
			}
		}
		String err = smd5sMap.getErrors();
		if ( err.length() > 0 ) {
			errors = addError( errors, err );
		}
		if ( errors.length() > 0 ) {
			resultTag = "FAILED";
		}
		if ( detailed ) {
			if ( errors.length() > 0 ) {
				is.out.println( "#ERROR:\n" + errors );
			}
			is.out.println( "#RESULT:" );
			is.out.println( Helper.readTextFile( tempDir + "/self.ls" ) );
			is.out.println( "#MD5s:" );
			is.out.println( smd5sMap.getOutput() );
		}
		else {
			is.out.println( "#USERID: " + is.regID );
			is.out.println( "#RESULT: " + resultTag );
			if ( errors.length() > 0 ) {
				is.out.println( "#ERROR: \n" + errors );
			}
		}
	}

	private String generateMD5s( String study, StudyMD5sMap smd5sMap ) throws IOException {
		File studyDicomDir = new File( RepositoryHandler.getLocation( RepositoryHandler.DATA, study ) );
		IMProperties ssr = new IMProperties();
		ssr.recRead( studyDicomDir.getAbsolutePath() + "/.info/storestate.rec" );
		if ( ssr.getProperty( "ProcessMode", "Deleted" ).equals( "Deleted" ) ) {
			return "";
		}
		String result = "";

		String[] cmd = new String[ 3 ];
		cmd[ 0 ] = IMEnvBase.getEnv( "DICOM_HOME" ) + "/bin/objdiff.sh";
		cmd[ 1 ] = "-m";

		String[] objectFiles = studyDicomDir.list();
		debug( "objectfiles:" + Helper.array2String( objectFiles ) );
		StudyMD5s smd5s = smd5sMap.getStudyMD5s( study );
		for ( int i = 0; i < objectFiles.length; i++ ) {
			if ( objectFiles[ i ].startsWith( "." ) ) {
				continue;
			}
			File objFile = new File( studyDicomDir, objectFiles[ i ] );
			if ( !objFile.isFile() ) {
				continue;
			}
			cmd[ 2 ] = objFile.getAbsolutePath();
			String[] env = { "HOME=" + IMEnvBase.getEnv( "HOME" ) };
			RunCommandResult rcr = Helper.runCommandEx( cmd, env );
			if ( rcr.getExitCode() != 0 ) {
				result += "\n objdiff.sh failed for file " + cmd[ 2 ];
				debug( "objdiff.sh failed for file " + cmd[ 2 ] );
			}
			else {
				String md5 = rcr.getOutput().trim();
				debug( "md5 of " + cmd[ 2 ] + " is " + md5 );
				smd5s.setProperty( objFile.getName(), md5, IMEnvBase.getEnv( "SERVERNAME" ) ); // there mustn't be any errors...
			}
		}
		if ( result.length() > 0 ) {
			result = result.substring( 1 ); // cut trailing "\n";
		}
		return result;
	}%><%IMSession is = new IMSession( request, response, out );

	is.response.setContentType( "text/plain" );

	if ( "true".equals( request.getParameter( "cFromRoot" ) ) ) {
		String parent = imagemed.dotcom.DotcomConfig.selfParent();
		if ( parent != null ) {
			try {
				String queryString = request.getQueryString();
				ResultSet rs = IMEnv.getDB().prepareStatement( "select HOST from " + IMEnvBase.getEnv( "TARGETTBL" ) + " where ID='" + parent + "'" ).executeQuery();
				String parentIp = rs.next() ? rs.getString( "HOST" ) : null;
				if ( parentIp == null ) {
					throw new imagemed.exception.IMException( "No entry in the devices table for parent server: " + parent );
				}
				rs.close();
				URL thepage = new URL( "https://" + parentIp + "/tools/checkStudies.jsp?" + queryString );
				HttpURLConnection conn = (HttpURLConnection) imagemed.http.IMProxy.openConnection( thepage, (String) null, null );

				InputStream inStream = conn.getInputStream();
				BufferedReader br = new BufferedReader( new InputStreamReader( inStream ) );
				try {
					String line = br.readLine();
					while ( line != null ) {
						out.println( line );
						line = br.readLine();
					}
				}
				catch ( IOException ioe ) {
				}
				inStream.close();
				conn.disconnect();
			}
			catch ( Throwable th ) {
				IfaceToolbox.printError( is, "Proxying to " + parent + " failed: " + Helper.throwable2String( th ) );
			}
			return;
		}
	}

	SynchronizedProperties childErrors = new SynchronizedProperties();
	Counter counter = new Counter();
	StudyMD5sMap smd5sMap = new StudyMD5sMap();

	String errors = "";

	try {
		if ( !IfaceToolbox.isRegOK( is ) && null == DotcomToolbox.checkClient( is ) )
			throw new Throwable();
	}
	catch ( Throwable th ) {
		is.out.println( "#USERID: " + is.regID );
		is.out.println( "#RESULT: FAILED" );
		is.out.println( "#ERROR: Has no registration info for this IP. (" + request.getRemoteAddr() + ", " + request.getServerName() + ")" );
		return;
	}

	IMRequest reqPar = new IMRequest( request );

	boolean detailed = "true".equals( request.getParameter( "cDetailed" ) );

	String tempDir = IMEnvBase.getEnv( "TMP" ) + "/getstates." + Helper.getCounter();

	String[] studies = null;

	if ( reqPar.exist( "cStudies" ) ) {
		studies = reqPar.getVals( "cStudies" );
	}
	else if ( !"true".equals( reqPar.getVal( "cAll" ) ) ) {
		errors = addError( errors, "Required parameter missing (cStudies or cAll=true)!" );
		generateOutput( is, detailed, smd5sMap, childErrors, errors, tempDir );
		return;
	}

	List<String> studyList = new ArrayList<String>();
	String studiesPar = "&cAll=true";
	if ( studies != null ) {
		studiesPar = "&cStudies=";
		for ( int i = 0; i < studies.length; i++ ) {
			studiesPar += studies[ i ] + ";";
		}
		studiesPar = studiesPar.substring( 0, studiesPar.length() - 1 ); // no need for tailing ';'

		StringTokenizer st = new StringTokenizer( studiesPar, "=;" );
		st.nextToken(); // "&cStudies"
		while ( st.hasMoreTokens() ) {
			String nechste = st.nextToken();
			studyList.add( nechste );
		}
	}

	Boolean2 isLocalReady = new Boolean2( false );
	ResultSet rs = null;
	Map<String, Thread> childCheckers = new HashMap<String, Thread>();
	try {
		// starting remote threads
		Map<String, Object> environment = new HashMap<String, Object>();
		environment.put( COUNTER, counter );
		environment.put( CHILDERRORS, childErrors );
		environment.put( STUDIESPAR, studiesPar );
		environment.put( SMD5SMAP, smd5sMap );
		environment.put( ISLOCALREADY, isLocalReady );
		environment.put( TEMPDIR, tempDir );
		rs = IMEnv.getDB().createStatement().executeQuery( "select ID from " +
				IMEnvBase.getEnv( "TARGETTBL" ) + " where TYPE='C'" );
		while ( rs.next() ) {
			String childID = rs.getString( "ID" );
			debug( "starting task for child " + childID );
			Thread t = new Thread( new ChildChecker( childID, environment ) );
			childCheckers.put( childID, t );
			t.start();
		}
		debug( "counting local MD5s" );
		// count local MD5s...
		String[] localStudies = RepositoryHandler.listDirs( RepositoryHandler.DATA );
		for ( int i = 0; i < localStudies.length; i++ ) {
			if ( studyList.isEmpty() || studyList.contains( localStudies[ i ] ) ) {
				errors = addError( errors, generateMD5s( localStudies[ i ], smd5sMap ) ); // this study may be Deleted...
			}
		}
		debug( "calling getstates.sh" );
		String[] cmd = new String[ studyList.size() + ( null != request.getHeader( "referer" ) ? 5 : 3 ) ];

		cmd[ 0 ] = IMEnvBase.getEnv( "TOOLS_HOME" ) + "/getstates.sh";
		cmd[ 1 ] = "-o";
		cmd[ 2 ] = tempDir;
		if ( null != request.getHeader( "referer" ) ) {
			cmd[ 3 ] = "-h";
			cmd[ 4 ] = request.getHeader( "referer" ).split( "/" )[ 2 ];
			//			IMEnv.debugLog(cmd[4]);
		}
		////
		//		Enumeration headerNames = request.getHeaderNames();
		//		
		//		while(headerNames.hasMoreElements()) {
		//			String headerName = (String)headerNames.nextElement();
		//			IMEnv.debugLog("headerName=" + headerName + "=" + request.getHeader(headerName));
		//		}
		////
		for ( int i = 0; i < studyList.size(); i++ ) {
			cmd[ i + ( null != request.getHeader( "referer" ) ? 5 : 3 ) ] = studyList.get( i );
		}
		String[] env = { "HOME=" + IMEnvBase.getEnv( "HOME" ) };
		RunCommandResult rcr = Helper.runCommandEx( cmd, env );
		if ( rcr.getExitCode() != 0 ) {
			errors = addError( errors, "getstates.sh failed: " + rcr.getError() );
			generateOutput( is, detailed, smd5sMap, childErrors, errors, tempDir );
			return;
		}
		debug( "negating isLocalReady" );
		// notify remote threads that the local MD5s are ready
		isLocalReady.negate();

		while ( counter.getValue() > 0 ) {
			Thread.sleep( 100 );
			debug( "counter.getValue=" + counter.getValue() );
		}
	}
	catch ( SQLException se ) {
		errors = addError( errors, "Database failed during querying of children!" );
	}
	debug( "generating final output" );
	generateOutput( is, detailed, smd5sMap, childErrors, errors, tempDir );%>