#!/bin/bash
#
# Component: tools
#
# Description:
#  Updates IP addresses in storestate.rec files.
#  Should be called after IP address changes of Parent/Child servers in dotcom.
#
# Usage:
#   ipchanged.sh <old IP> <new IP>
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="ipchanged[$$]"


oldIP=$1
newIP=$2

if [ -z "$newIP" ] ; then
  $LOGGER -p $LOG_FACILITY.err -t "$TAG" "Missing parameters"
  exit 1
fi

$LOGGER -p $LOG_OPER_FACILITY.info -t "$TAG" "INFO OTHER ipchanged system - - - - - - - - - - - IP_address_changed_(${oldIP}_to_${newIP})"
$LOG_HOME/logintodatabase.sh -p "INFO" -l "OTHER" -a "ipchanged" -u "system" -m "IP_address_changed_(${oldIP}_to_${newIP})" 

IPSAVEDIR=$TMP/ipsave.$$

# create directory for backing up storestate.rec files temporarily
mkdir $IPSAVEDIR 2>/dev/null
[ ! -d "$IPSAVEDIR" ] && exit 1

CWD=`pwd`

[ -z "$REPOSITORY_DICOM_DATA" ] && exit 1


for study in `$REPOSITORYHANDLER_HOME/bin/listDirectories $REPOSITORY_DATA_DICOM` ; do
	studydir=`$REPOSITORYHANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $study`
	cd $studydir
	cd ..

  $LOGGER -p $LOG_FACILITY.info -t "$TAG" "Updating storestate.rec for study $study"

  # save storestate.rec file first
  cp -f -p $study/storestate.rec $IPSAVEDIR/$study.rec

  if [ ! -f "$IPSAVEDIR/$study.rec" ] ; then
    $LOGGER -p $LOG_FACILITY.err -t "$TAG" "ERROR could not update and create backup file of storestate.rec file of study $study"
    continue
  fi

  #replace oldIP occurences in storestate.rec to newIP (creating new file)
  sed "s/${oldIP}/${newIP}/g" $IPSAVEDIR/$study.rec > $IPSAVEDIR/$study_new.rec
  
  chmod a-w $IPSAVEDIR/$study_new.rec

  # overwrite storestate.rec with updated file
  mv -f $IPSAVEDIR/$study_new.rec $study/storestate.rec

  [ $? -ne 0 ] && $LOGGER -p $LOG_FACILITY.err -t "$TAG" "ERROR updating storestate.rec for study $study"

done

cd $CWD

$LOGGER -p $LOG_OPER_FACILITY.info -t "$TAG" "INFO OTHER ipchanged system - - - - - - - - - - - Applying_IP_address_change_finished"
$LOG_HOME/logintodatabase.sh -p "INFO" -l "OTHER" -a "ipchanged" -u "system" -m "Applying_IP_address_change_finished" 
