#!/bin/gawk -f 
#
# Extracts and prints crash log headers from viewerinfo log files.
#


#
# 
#
function reset() {
	clientInfo="";
	isHeader=1;
	isStudies=0;
	isEPViewer=0;
}

BEGIN {
	isHeader = 0;
	isFirst = 1;
}


# {
#	print "DEBUG:[" $1 "]"
# }


( $1 == "@@@" && $4 == "Viewer" && $5 == "Info" ) {
	if ( isFirst ) {
		isFirst = 0;
	}
	else {
		print "------------";
		print "";
	}
	reset();
	next;
}


/^### ViewerClient: / {
	if( !isHeader ) {
		next;
	}

	clientInfo=gensub( "### ViewerClient: ", "", "1" );
	next;
}


/^Crash time: / {
	if ( !isHeader ) {
		next;
	}

	printf "%-20s %s %s\n", "Crash time:", $3, $4;
	next;
}


/^eRAD PACS Viewer/ {
	if (!isHeader) {
		next;
	}

	isEPViewer=1;
	next;
}


/^Version:? / {
	if ( !isEPViewer ) {
		next;
	}

	isEPViewer=0;

	printf "%-20s %s\n", "Viewer Version:", gensub( "^Version:? ", "", "1" );
	if ( clientInfo != "" ) {
		printf "%-20s %s\n", "Viewer Client:", clientInfo
	};
	next;
}


# ( $1 == "SymInit:" && $6 == "UserName:" ) { if ( !isHeader ) next; printf "%-20s %s\n", "Windows User:", $7; next; } 

/^OS-Version: / {
	if ( !isHeader ) {
		next;
	}

	printf "%-20s %s\n", "OS-Version:", gensub( "^OS-Version: ", "", "1" );
	next;
}


/^Connected server:/ {
	if ( !isHeader ) {
		next;
	}

	printf "%-20s %s\n", "Connected server:", gensub( "^Connected server: ", "", "1" );
	next;
}


/^Prefetch server:/ {
	if ( !isHeader ) {
		next;
	}

	printf "%-20s %s\n", "Prefetch server:", gensub( "^Prefetch server: ", "", "1" );
	next;
}


/^Opened studies:/ {
	if ( !isHeader ) {
		next;
	}

	isStudies=1;
	print;
	next;
}


( $1 == "" ) {
	if (isStudies) {
		print;
		isHeader=0;
		isStudies=0;
		isEPViewer=0;
	};

	next;
}


( isStudies ) {
	printf "\t"; print;
}

