#!/bin/bash
#
# Component: tools
#
# Description:
#   Stat post-processing module for objects processing latency
#
# Usage:
#
#    post_latency.sh [options]
#
#	-f returns the logfile used
#	-p returns the stat properties it creates
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$


if [ "$1" == "-f" ] ; then
  echo info.log
  exit
elif [ "$1" == "-p" ] ; then
  echo "obj_processing_latency
obj_processing_behind"
  exit
fi

if [ "$1" == "-d" ] ; then
  gdate=`date -d $2 +"%b %e"`
else
  gdate=`date +"%b %e"`
fi

grep -a -e "acquire" -e "retrieve" -e "Stored" -e "(Similar" | grep -a "^$gdate" | gawk '

function difftime(a,b) {
        split(a,aa,":");
        split(b,bb,":");
        return (bb[1]*3600+bb[2]*60+bb[3] - aa[1]*3600-aa[2]*60-aa[3]);
}

function doProc(oo, end)
{
  if (oo in tm)
  {
     start=tm[oo];
#     print "#Object: " start " - " end;
     split(end,cma,":");
     split(start,lma,":");
#     print lma[1] ":" lma[2] "-" cma[1] ":" cma[2];

     lat=-1;

     for (h=lma[1];h<=cma[1];h++)
       for (m= (h> 0+lma[1] ? 0 : lma[2]+1) ; m< (h<0+cma[1] ? 60 : cma[2]+1) ;m++) {
       t=sprintf("%02d:%02d",h,m);
       if (lat==-1) lat=difftime(start,t ":00");
       else lat=lat+60;
       mmo[t]++;
       if (lat>mml[t]){
	 mml[t]=lat;
#         print "# latency at " t " is now " lat;
       }
     }
    delete tm[oo];
  }
}

BEGIN { lm="00:00"; q=0; oldest="99:99:99"; } 

$6=="SCP" && $7=="Stored" && \
  $8 !~"^PbR" && $8!~"^PbD"  && $8!~ "^SRt"  {
  # add
  if (! ($8 in tm))
  {
    tm[$8]=$3;
  }
}


$5 == "dcreg:" && $7=="INFO" && $8=="OBJECT" && ($9=="acquire" || $9="retrieve") && \
 $11!~"^PbR" && $11 !~"^PbD" && $11 !~"^SRt"  {
   doProc($10,$3); 
}

$5 ~ /dcreg\[.+\]:/ && $6=="INFO" && $7=="OBJECT" && ($8=="acquire" || $8="retrieve") && \
 $10!~"^PbR" && $10 !~"^PbD" && $10 !~"^SRt"  {
   doProc($10,$3); 
}

$8=="(Similar" && $6!~"^PbR" && $6 !~"^PbD" && $6 !~"^SRt"  {
   doProc($6,$3); 
}


END {
  for (h=0;h<24;h++) for (m=0;m<60;m++) {
    t=sprintf("%02d:%02d",h,m);
    printf("monitorSTART=xxxx/xx/xx %s:00\n",t);
    printf("obj_processing_latency=%d\n",mml[t]);
    printf("obj_processing_behind=%d\n",mmo[t]);
    }

}

'