#!/bin/bash
#
# Component: util
#
# Description:
#  Compresses the whole Archive.
#
# Usage:
#   compressArchive.sh 
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#

. $HOME/etc/setenv.sh
isFolder()
{
    currentDirectory=`pwd`
    cd "$1" 2>/dev/null
    [ $? != 0 ] && return 1
    cd -
    return 0
}

compressDir_()
{
    if [ -f $1 ] ; then
	echo "${tabulation}$UTILS_HOME/bin/compressObject.sh -i $1"
	$UTILS_HOME/bin/compressObject.sh -i $1 -v -t "..$tabulation" 2>/dev/null
	return $?
    fi
    isFolder $1
    [ $? != 0 ] && return 0
    cd "$1"
    echo "${tabulation}processing $1"
    tabulation="..$tabulation"
    for i in `ls 2>/dev/null` ; do
	compressDir_ $i
    done
    tabulation="${tabulation:2}"
    cd ..
}

[ -f "$ARCHIVE_VAR/archive.cfg" ] && . $ARCHIVE_VAR/archive.cfg

if [ "$Archive" != "yes" ] ; then
    echo "There's no active Archive."
    exit 1
fi

if [ -z "$CompressionMode" -o "none" == "$CompressionMode" ] ; then
    echo "The archive is not configured to use compression."
    exit 1
fi

startPoint=`pwd`

if [ ! -d "$ARCHIVE_DIR/$MountPoint" ] ; then
    echo "The Archive mount point doesn't exist."
    exit 1
fi

[ -z "$ArchiveTempDir" ] && ArchiveTempDir="$ARCHIVE_TEMPDIR"

if [ ! -d "$ArchiveTempDir" ] ; then
    echo "The Archive temp dir doesn't exist."
    exit 1
fi

echo "You are about to compress the Archive using the following parameters:"
echo "  Archive root = $ARCHIVE_DIR/$MountPoint"
echo "  Archive temp dir = $ArchiveTempDir"
echo "  Compression = $CompressionMode"
echo "The process should run safely under running server, but to ensure safety we recommend to stop the server while this script runs."
echo "Enter 'yes' to continue!"
read answer
if [ "$answer" != "yes" ] ; then
    echo "terminated..."
    exit 1
fi

startDir=`pwd`

tabulation=".."

echo "compressing archive..."
compressDir_ "$ARCHIVE_DIR/$MountPoint"
echo "compressing archive temp..."
compressDir_ "$ArchiveTempDir"
cd "$startDir"

echo "finished."
